var electron = require('electron');
var path = require('path');
const os = require('os');
const url = require('url')

const isLinux = os.platform() === 'linux';

var app = electron.app;

var Menu = electron.Menu;
var BrowserWindow = electron.BrowserWindow;

var close = false;
var mainWindow = null;
var base = path.join(__dirname, 'index.html');
const PROTOCOL = 'igap://';

function formFullTodoUrl(path) {
    if (isLinux || !path || !path.startsWith(PROTOCOL)) {
        return base
    }
    return base + '?igaddr=' + path;
}

// initialization and is ready to create browser windows.
function createWindow() {

    // remove app default menu
    Menu.setApplicationMenu(null);

    if (makeSingleInstance()) {
        return;
    }
    setupWindows();
    if (!isLinux) {
        setupTray();
        handleSquirrelEvent();
    }
}
function setupWindows() {

    // Create the browser window.
    mainWindow = new BrowserWindow({
        width: 1000,
        height: 625,
        minWidth: 800,
        minHeight: 625,
        icon: path.join(__dirname, 'favicon.png')
    });
    mainWindow.setOverlayIcon(path.join(__dirname, 'favicon.png'), "iGap-desktop");
    mainWindow.on('close', function (event) {
        if (!close) {
            mainWindow.hide();
            event.preventDefault();
            return false;
        }
    });
    mainWindow.on('closed', function () {
        mainWindow = null;
    });

    const firstUrl = formFullTodoUrl(process.argv[1]);
    if (!isLinux) {
        mainWindow.loadURL(firstUrl);
    } else {
        mainWindow.loadURL(url.format({
            pathname: firstUrl,
            protocol: 'file:',
            slashes: true
        }));
    }

    // Close app whene `f10` press
    electron.globalShortcut.register('CommandOrControl+F10', function () {
        var result = electron.dialog.showMessageBox({
            type: 'warning',
            title: 'iGap Alert',
            buttons: ['Cancel', 'Exit Now'],
            message: 'Exit Alert!',
            cancelId: 0,
            detail: 'Are you sure exiting iGap desktop client ?'
        });
        if (result !== 0) {
            close = true;
            mainWindow.close()
        }
    });
}

function setupTray() {
    var tray = new electron.Tray(path.join(__dirname, 'favicon.png'));
    tray.setToolTip('iGap desktop application.');
    tray.setContextMenu(Menu.buildFromTemplate([
        {
            label: 'Open', click: function () {
                mainWindow.show()
            }
        },
        {
            label: 'Quit', click: function () {
                close = true;
                mainWindow.close()
            }
        }
    ]));

    tray.on('click', function () {
        mainWindow.isVisible() ? mainWindow.hide() : mainWindow.show()
    });
    mainWindow.on('show', function () {
        tray.setHighlightMode('always')
    });
    mainWindow.on('hide', function () {
        tray.setHighlightMode('never')
    });
}

function makeSingleInstance() {
    return app.makeSingleInstance(function (commandLine, workingDirectory) {
        // Someone tried to run a second instance, we should focus our window
        if (mainWindow) {
            if (mainWindow.isMinimized()) mainWindow.restore();
            mainWindow.show();
            mainWindow.focus();
            const firstUrl = formFullTodoUrl(commandLine[1]);
            if (firstUrl !== base) {
                mainWindow.webContents.send('CHECK_LOCATION_TG_ADDR', commandLine[1])
            }
        }
        return true;
    });
}

function handleSquirrelEvent() {
    if (isLinux || process.argv.length === 1) {
        return false;
    }

    const ChildProcess = require('child_process');

    const appFolder = path.resolve(process.execPath, '..');
    const rootAtomFolder = path.resolve(appFolder, '..');
    const updateDotExe = path.resolve(path.join(rootAtomFolder, 'Update.exe'));
    const exeName = path.basename(process.execPath);

    const spawn = function (command, args) {
        var spawnedProcess, error;

        try {
            spawnedProcess = ChildProcess.spawn(command, args, {detached: true});
        } catch (error) {
        }

        return spawnedProcess;
    };

    const spawnUpdate = function (args) {
        return spawn(updateDotExe, args);
    };

    const squirrelEvent = process.argv[1];
    switch (squirrelEvent) {
        case '--squirrel-install':
        case '--squirrel-updated':
            // Optionally do things such as:
            // - Add your .exe to the PATH
            // - Write to the registry for things like file associations and
            //   explorer context menus

            // Install desktop and start menu shortcuts
            spawnUpdate(['--createShortcut', exeName]);

            setTimeout(app.quit, 1000);
            return true;

        case '--squirrel-uninstall':
            // Undo anything you did in the --squirrel-install and
            // --squirrel-updated handlers

            // Remove desktop and start menu shortcuts
            spawnUpdate(['--removeShortcut', exeName]);

            setTimeout(app.quit, 1000);
            return true;

        case '--squirrel-obsolete':
            // This is called on the outgoing version of your app before
            // we update to the new version - it's the opposite of
            // --squirrel-updated

            app.quit();
            return true;
    }
}

app.setAppUserModelId("net.rooyekhat.igap.desktop");

app.on('ready', createWindow);

// Quit when all windows are closed.
app.on('window-all-closed', function () {
    // On OS X it is common for applications and their menu bar
    if (process.platform != 'darwin') {
        app.quit();
    }
});
